@@ -6,87 +6,93 @@
 #ifdef KHICAS
 #include "misc.h"
 
+#include <exception>
+
+class autoshutdown : public std::exception{
+  const char * what () const throw ()
+  {
+    return "autoshutdown";
+  }
+};
+
 extern  const int LCD_WIDTH_PX;
 extern   const int LCD_HEIGHT_PX;
+extern char* fmenu_cfg;
 #define STATUS_AREA_PX 0 // 24
 #define GIAC_HISTORY_MAX_TAILLE 32
 #define GIAC_HISTORY_SIZE 2
 
-// next 3 functions may be void if not inside a window class hierarchy
-void os_show_graph(); // show graph inside Python shell (Numworks), not used
-void os_hide_graph(); // hide graph, not used anymore
-void os_redraw(); // force redraw of window class hierarchy
-#ifdef NUMWORKS
-void numworks_set_pixel(int x,int y,int c);
-int numworks_get_pixel(int x,int y);
-void numworks_fill_rect(int x,int y,int w,int h,int c);
-int numworks_draw_string(int x,int y,int c,int bg,const char * s,bool fake=false);
-int numworks_draw_string_small(int x,int y,int c,int bg,const char * s,bool fake=false);
-void numworks_show_graph();
-void numworks_hide_graph();
-void numworks_redraw();
-void numworks_wait_1ms(int ms);
-// access to Numworks OS, defined in port.cpp (or modkandinsky.cpp)
-inline void os_set_pixel(int x,int y,int c){
-  numworks_set_pixel(x,y,c);
+#ifdef MICROPY_LIB
+extern "C" {
+  int do_file(const char *file);
+  char * micropy_init(int stack_size,int heap_size);
+  int micropy_eval(const char * line);
+  void  mp_deinit();
+  void mp_stack_ctrl_init();
+  extern int parser_errorline,parser_errorcol;
 }
-inline int os_get_pixel(int x,int y){
-  return numworks_get_pixel(x,y);
-}
-inline void os_fill_rect(int x,int y,int w,int h,int c){
-  numworks_fill_rect(x,y,w,h,c);
-}
-inline int os_draw_string(int x,int y,int c,int bg,const char * s,bool fake){
-  return numworks_draw_string(x,y,c,bg,s,fake);
-}
-inline int os_draw_string_small(int x,int y,int c,int bg,const char * s,bool fake){
-  return numworks_draw_string_small(x,y,c,bg,s,fake);
-}
-inline void os_shaw_graph(){ return numworks_show_graph(); }
-inline void os_hide_graph(){ return numworks_hide_graph(); }
-inline void os_redraw(){ return numworks_redraw(); }
-inline void os_wait_1ms(int ms) { numworks_wait_1ms(ms); }
-#endif // NUMWORKS
+int micropy_ck_eval(const char *line);
+#endif
 
-bool os_set_angle_unit(int mode);
-int os_get_angle_unit();
-double millis(); extern int time_shift;
-void os_wait_1ms();
-bool file_exists(const char * filename);
-bool erase_file(const char * filename);
-const char * read_file(const char * filename);
-bool write_file(const char * filename,const char * s,size_t len=0);
-#define MAX_NUMBER_OF_FILENAMES 255
-int os_file_browser(const char ** filenames,int maxrecords,const char * extension);
-void os_set_pixel(int x,int y,int c);
-void os_fill_rect(int x,int y,int w,int h,int c);
-inline void drawRectangle(int x,int y,int w,int h,int c){
-  os_fill_rect(x,y,w,h,c);
+#include "k_csdk.h"
+
+extern "C" {
+  int do_shutdown(); // auto-shutdown
+  void console_output(const char *,int );
+  const char * console_input(const char * msg1,const char * msg2,bool numeric,int ypos);
+  void c_draw_rectangle(int x,int y,int w,int h,int c);
+  void c_fill_rect(int x,int y,int w,int h,int c);
+  void c_draw_line(int x0,int y0,int x1,int y1,int c);
+  void c_draw_circle(int xc,int yc,int r,int color,bool q1,bool q2,bool q3,bool q4);
+  void c_draw_filled_circle(int xc,int yc,int r,int color,bool left,bool right);
+  void c_draw_polygon(int * x,int *y ,int n,int color);
+  void c_draw_filled_polygon(int * x,int *y, int n,int xmin,int xmax,int ymin,int ymax,int color);
+  void c_draw_arc(int xc,int yc,int rx,int ry,int color,double theta1, double theta2);
+  void c_draw_filled_arc(int x,int y,int rx,int ry,int theta1_deg,int theta2_deg,int color,int xmin,int xmax,int ymin,int ymax,bool segment);
+  void c_set_pixel(int x,int y,int c);
+  int c_draw_string(int x,int y,int c,int bg,const char * s,bool fake);
+  int c_draw_string_small(int x,int y,int c,int bg,const char * s,bool fake);
+  int c_draw_string_medium(int x,int y,int c,int bg,const char * s,bool fake);
+  int select_item(const char ** ptr,const char * title,bool askfor1=true);
+  // C conversion to gen from atomic data type 
+  unsigned long long c_double2gen(double); 
+  unsigned long long c_int2gen(int);
+  // linalg on double matrices
+  void doubleptr2matrice(double * x,int n,giac::matrice & m);
+  bool matrice2doubleptr(const giac::matrice &M,double *x); // x must have enough space
+  bool r_inv(double *,int n);
+  bool r_rref(double *,int n,int m);
+  double r_det(double *,int);
+  struct double_pair {
+    double r,i;
+  } ;
+  typedef struct double_pair c_complex;
+  bool matrice2c_complexptr(const giac::matrice &M,c_complex *x);
+  void c_complexptr2matrice(c_complex * x,int n,int m,giac::matrice & M);  
+  bool c_inv(c_complex *,int n);
+  bool c_rref(c_complex *,int n,int m);
+  c_complex c_det(c_complex *,int);
+  bool c_egv(c_complex * x,int n); // eigenvectors
+  bool c_eig(c_complex * x,c_complex * d,int n); // x eigenvect, d reduced mat
+  bool c_proot(c_complex * x,int n); // poly root
+  bool c_pcoeff(c_complex * x,int n); // root->coeffs
+  bool c_fft(c_complex * x,int n,bool inverse); // FFT
+  void turtle_freeze();
+  void c_sprint_double(char * s,double d);
+  extern int python_stack_size,python_heap_size;
+  extern int xcas_python_eval;
+  extern char * python_heap;
+  int python_init(int stack_size,int heap_size);
 }
-int os_get_pixel(int x,int y);
-/* returns new x position */
-int os_draw_string(int x,int y,int c,int bg,const char * s,bool fake=false);
-inline int os_draw_string(int x,int y,const char * s){ return os_draw_string(x,y,giac::_BLACK,giac::_WHITE,s);}
-int os_draw_string_small(int x,int y,int c,int bg,const char * s,bool fake=false);
-inline int os_draw_string_small(int x,int y,const char * s){ return os_draw_string_small(x,y,giac::_BLACK,giac::_WHITE,s);}
-void GetKey(int * key);
-int getkey_raw(bool allow_suspend); // Numworks scan code
-int getkey(bool allow_suspend); // transformed
-void enable_back_interrupt();
-inline void set_abort(){  enable_back_interrupt(); }
-void disable_back_interrupt();
-inline void clear_abort(){  disable_back_interrupt(); }
-bool isalphaactive();
-extern bool alphawasactive;
-void lock_alpha();
-void reset_kbd();
-void statuslinemsg(const char * msg);
-void statusline(int mode=0);
-int select_item(const char ** ptr,const char * title);
+extern int lang;
+extern bool warn_nr;
+int select_interpreter(); // 0 Xcas, 1|2 Xcas python_compat(1|2), 3 MicroPython 
+const char * gettext(const char * s) ;
 
 #ifndef NO_NAMESPACE_XCAS
 namespace xcas {
 #endif // ndef NO_NAMESPACE_XCAS
+  void set_exam_mode(int i,const giac::context *);
   int giac_filebrowser(char * filename,const char * extension,const char * title);
   void draw_rectangle(int x,int y,int w,int h,int c);
   void draw_line(int x0,int y0,int x1,int y1,int c);
@@ -96,6 +102,10 @@ namespace xcas {
   void draw_filled_polygon(std::vector< vector<int> > &L,int xmin,int xmax,int ymin,int ymax,int color);
   void draw_arc(int xc,int yc,int rx,int ry,int color,double theta1, double theta2);
   void draw_filled_arc(int x,int y,int rx,int ry,int theta1_deg,int theta2_deg,int color,int xmin,int xmax,int ymin,int ymax,bool segment);
+
+  //syntax colored printing
+  int print_color(int print_x,int print_y,const char *s,int color,bool invert,bool minimini,const giac::context * contextptr);
+  bool tooltip(int x,int y,int pos,const char * editline,const giac::context * contextptr);
 			   
   bool textedit(char * s,int bufsize,const giac::context * contextptr);
   // maximum "size" of symbolics displayed in an Equation (pretty print)
@@ -162,11 +172,12 @@ namespace xcas {
 #endif
     int turtlex,turtley; // Turtle translate
     double turtlezoom; // Zoom factor for turtle screen
-    int maillage; // 0 (none), 1 (square), 2 (triangle), bit3 used for printing
+    short int maillage=0; // 0 (none), 1 (square), 2 (triangle), bit3 used for printing
+    short int speed=0;
   };
   
-  void displaygraph(const giac::gen & ge, const giac::context * contextptr);
-  void displaylogo();
+  int displaygraph(const giac::gen & ge, const giac::context * contextptr);
+  int displaylogo();
   giac::gen eqw(const giac::gen & ge,bool editable,const giac::context * contextptr);
   typedef short int color_t;
   typedef struct
@@ -203,6 +214,7 @@ namespace xcas {
     bool changed=false;
     int python=0;
     int type=TEXTAREATYPE_NORMAL;
+    int cursorx,cursory; // where the last cursor was displayed
   } textArea;
 
 #define TEXTAREA_RETURN_EXIT 0
@@ -219,9 +231,10 @@ namespace xcas {
   void reload_edptr(const char * filename,textArea *edptr,const giac::context *);
   void print(int &X,int&Y,const char * buf,int color,bool revert,bool fake,bool minimini);
 
+  void save_session(const giac::context * );
 #if 1
-#define MAX_FILENAME_SIZE 270
-  void save_console_state_smem(const char * filename,const giac::context *);
+#define MAX_FILENAME_SIZE 63
+  void save_console_state_smem(const char * filename,bool xwaspy,const giac::context *);
   bool load_console_state_smem(const char * filename,const giac::context *);
 
   struct DISPBOX {
@@ -238,7 +251,7 @@ namespace xcas {
 			   CONSOLE_SUCCEEDED = 0,
 			   CONSOLE_MEM_ERR = -1,
 			   CONSOLE_ARG_ERR = -2,
-			   CONSOLE_NO_EVENT = -3
+			   CONSOLE_NO_EVENT = -3,
   };
 
   enum CONSOLE_CURSOR_DIRECTION{
@@ -265,7 +278,11 @@ namespace xcas {
   enum CONSOLE_SCREEN_SPEC {
 			    _LINE_MAX = 48,
 			    LINE_DISP_MAX = 11,
+#ifdef NSPIRE_NEWLIB
+			    COL_DISP_MAX = 32,
+#else
 			    COL_DISP_MAX = 30,//32
+#endif
 			    EDIT_LINE_MAX = 2048
   };
   
@@ -303,14 +320,14 @@ namespace xcas {
   int Console_Output(const char *str);
   void Console_Clear_EditLine();
   int Console_NewLine(int pre_line_type, int pre_line_readonly);
-  int Console_Backspace(void);
+  int Console_Backspace(const giac::context *);
   int Console_GetKey(const giac::context *);
-  int Console_Init(void);
-  int Console_Disp(int redraw_mode=1);
+  int Console_Init(const giac::context *);
+  int Console_Disp(int redraw_mode,const giac::context*ptr);
   int Console_FMenu(int key,const giac::context *);
   extern char menu_f1[8],menu_f2[8],menu_f3[8],menu_f4[8],menu_f5[8],menu_f6[8];
   const char * console_menu(int key,char* cfg,int active_app);
-  void Console_FMenu_Init(void);
+  void Console_FMenu_Init(const giac::context *);
   const char * Console_Draw_FMenu(int key, struct FMenu* menu,char * cfg_,int active_app);
   char *Console_Make_Entry(const char* str);
   char *Console_GetLine(const giac::context *);
@@ -330,11 +347,27 @@ namespace xcas {
 #endif
   int periodic_table(const char * & name,const char * & symbol,char * protons,char * nucleons,char * mass,char * electroneg);
 
+  struct tableur {
+    giac::matrice m,clip,undo;
+    giac::gen var;
+    int nrows,ncols;
+    int cur_row,cur_col,disp_row_begin,disp_col_begin;
+    int sel_row_begin,sel_col_begin;
+    std::string cmdline,filename;
+    int cmd_pos,cmd_row,cmd_col; // row/col of current cmdline, -1 if not active
+    bool changed,recompute,matrix_fill_cells,movedown,keytooltip;
+  } ;
+  extern tableur * sheetptr;
+
+  int check_do_graph(giac::gen & ge,int do_logo_graph_eqw,const giac::context *);
+  int get_filename(char * filename,const char * extension);
+  int find_color(const char * s,const giac::context *);
 
 #ifndef NO_NAMESPACE_XCAS
 } // namespace xcas
 #endif // ndef NO_NAMESPACE_XCAS
 
+giac::gen sheet(const giac::context *); // in kadd.cc
 /* ************************************************************
 **************************************************************
 ***********************************************************  */
@@ -343,6 +376,10 @@ namespace xcas {
 #ifndef NO_NAMESPACE_XCAS
 namespace giac {
 #endif // ndef NO_NAMESPACE_XCAS
+  // back: number of char that should be deleted,
+  // exec=0 or MENU_RETURN_SELECTION, KEY_CHAR_ANS or KEY_CTRL_EXE
+  std::string help_insert(const char * cmdline,int & back,int exec,const giac::context *);
+  void copy_clipboard(const std::string & s,bool status);
 #define TEXT_MODE_NORMAL 0
 #define TEXT_MODE_INVERT 1
 #define MENUITEM_NORMAL 0
@@ -353,14 +390,14 @@ namespace giac {
   typedef struct
   {
     char* text; // text to be shown on screen. mandatory, must be a valid pointer to a string.
-    int type=MENUITEM_NORMAL; // type of the menu item. use MENUITEM_* to set this
-    int value=MENUITEM_VALUE_NONE; // value of the menu item. For example, if type is MENUITEM_CHECKBOX and the checkbox is checked, the value of this var will be MENUITEM_VALUE_CHECKED
-    int color=giac::_BLACK; // color of the menu item (use TEXT_COLOR_* to define)
-    // The following two settings require the menu type to be set to MENUTYPE_MULTISELECT
-    int isfolder=0; // for file browsers, this will signal the item is a folder
-    int isselected=0; // for file browsers and other multi-select screens, this will show an arrow before the item
-    int icon=-1; //for file browsers, to show a file icon. -1 shows no icon (default)
     int token; // for syntax help on keywords not in the catalog
+    short int isfolder=0; // for file browsers, this will signal the item is a folder
+    signed char color=giac::_BLACK; // color of the menu item (use TEXT_COLOR_* to define)
+    signed char type=MENUITEM_NORMAL; // type of the menu item. use MENUITEM_* to set this
+    signed char value=MENUITEM_VALUE_NONE; // value of the menu item. For example, if type is MENUITEM_CHECKBOX and the checkbox is checked, the value of this var will be MENUITEM_VALUE_CHECKED
+    // The following two settings require the menu type to be set to MENUTYPE_MULTISELECT
+    signed char isselected=0; // for file browsers and other multi-select screens, this will show an arrow before the item
+    signed char icon=-1; //for file browsers, to show a file icon. -1 shows no icon (default)
   } MenuItem;
 
   typedef struct
@@ -381,7 +418,7 @@ namespace giac {
     char* nodatamsg; // message to show when there are no menu items to display
     int startX=1; //X where to start drawing the menu. NOTE this is not absolute pixel coordinates but rather character coordinates
     int startY=0; //Y where to start drawing the menu. NOTE this is not absolute pixel coordinates but rather character coordinates
-    int width=22; // NOTE this is not absolute pixel coordinates but rather character coordinates
+    int width=30; // NOTE this is not absolute pixel coordinates but rather character coordinates
     int height=12; // NOTE this is not absolute pixel coordinates but rather character coordinates
     int scrollbar=1; // 1 to show scrollbar, 0 to not show it.
     int scrollout=0; // whether the scrollbar goes out of the menu area (1) or it overlaps some of the menu area (0)
@@ -409,18 +446,25 @@ namespace giac {
     const char* desc;
     const char * example;
     const char * example2;
-    int category;
+    unsigned int category;
   } catalogFunc;
 
+  void aide2catalogFunc(const giac::aide & a,catalogFunc & c);
+
   giac::gen select_var(const giac::context * contextptr);
+  const char * keytostring(int key,int keyflag,bool py,const giac::context * contextptr);
+  void insert(std::string & s,int pos,const char * add);
+  
   int showCatalog(char* insertText,int preselect,int menupos,const giac::context * contextptr);
   int doMenu(Menu* menu, MenuItemIcon* icontable=NULL);
   void reset_alpha();
   // category=0 for CATALOG, 1 for OPTN
   // returns 0 on exit, 1 on success
   int doCatalogMenu(char* insertText, const char* title, int category,const giac::context * contextptr);
-  extern const char shortcuts_string[];
-  extern const char apropos_string[];
+  extern const char shortcuts_fr_string[];
+  extern const char shortcuts_en_string[];
+  extern const char apropos_fr_string[];
+  extern const char apropos_en_string[];
   void init_locale();
 
   gen turtle_state(const giac::context * contextptr);
@@ -455,174 +499,6 @@ namespace giac {
 #define TEXT_COLOR_WHITE giac::_WHITE
 #define TEXT_COLOR_MAGENTA giac::_MAGENTA
 
-// Character codes
-#define KEY_CHAR_0          0x30
-#define KEY_CHAR_1          0x31
-#define KEY_CHAR_2          0x32
-#define KEY_CHAR_3          0x33
-#define KEY_CHAR_4          0x34
-#define KEY_CHAR_5          0x35
-#define KEY_CHAR_6          0x36
-#define KEY_CHAR_7          0x37
-#define KEY_CHAR_8          0x38
-#define KEY_CHAR_9          0x39
-#define KEY_CHAR_DP         0x2e
-#define KEY_CHAR_EXP        0x0f
-#define KEY_CHAR_PMINUS     30200
-#define KEY_CHAR_PLUS       43
-#define KEY_CHAR_MINUS      45
-#define KEY_CHAR_MULT       42
-#define KEY_CHAR_DIV        47
-#define KEY_CHAR_FRAC       0xbb
-#define KEY_CHAR_LPAR       0x28
-#define KEY_CHAR_RPAR       0x29
-#define KEY_CHAR_COMMA      0x2c
-#define KEY_CHAR_STORE      0x0e
-#define KEY_CHAR_LOG        0x95
-#define KEY_CHAR_LN         0x85
-#define KEY_CHAR_SIN        0x81
-#define KEY_CHAR_COS        0x82
-#define KEY_CHAR_TAN        0x83
-#define KEY_CHAR_SQUARE     0x8b
-#define KEY_CHAR_POW        0xa8
-#define KEY_CHAR_IMGNRY     0x7f50
-#define KEY_CHAR_LIST       0x7f51
-#define KEY_CHAR_MAT        0x7f40
-#define KEY_CHAR_EQUAL      0x3d
-#define KEY_CHAR_PI         0xd0
-#define KEY_CHAR_ANS        0xc0
-#define KEY_SHIFT_ANS        0xc1
-#define KEY_CHAR_LBRCKT     0x5b
-#define KEY_CHAR_RBRCKT     0x5d
-#define KEY_CHAR_LBRACE     0x7b
-#define KEY_CHAR_RBRACE     0x7d
-#define KEY_CHAR_CR         0x0d
-#define KEY_CHAR_CUBEROOT   0x96
-#define KEY_CHAR_RECIP      0x9b
-#define KEY_CHAR_ANGLE      0x7f54
-#define KEY_CHAR_EXPN10     0xb5
-#define KEY_CHAR_EXPN       0xa5
-#define KEY_CHAR_ASIN       0x91
-#define KEY_CHAR_ACOS       0x92
-#define KEY_CHAR_ATAN       0x93
-#define KEY_CHAR_ROOT       0x86
-#define KEY_CHAR_POWROOT    0xb8
-#define KEY_CHAR_SPACE      0x20
-#define KEY_CHAR_DQUATE     0x22
-#define KEY_CHAR_VALR       0xcd
-#define KEY_CHAR_THETA      0xce
-#define KEY_CHAR_FACTOR     0xda
-#define KEY_CHAR_NORMAL     0xdb
-#define KEY_CHAR_SHIFTMINUS     0xdc
-#define KEY_CHAR_A          0x41
-#define KEY_CHAR_B          0x42
-#define KEY_CHAR_C          0x43
-#define KEY_CHAR_D          0x44
-#define KEY_CHAR_E          0x45
-#define KEY_CHAR_F          0x46
-#define KEY_CHAR_G          0x47
-#define KEY_CHAR_H          0x48
-#define KEY_CHAR_I          0x49
-#define KEY_CHAR_J          0x4a
-#define KEY_CHAR_K          0x4b
-#define KEY_CHAR_L          0x4c
-#define KEY_CHAR_M          0x4d
-#define KEY_CHAR_N          0x4e
-#define KEY_CHAR_O          0x4f
-#define KEY_CHAR_P          0x50
-#define KEY_CHAR_Q          0x51
-#define KEY_CHAR_R          0x52
-#define KEY_CHAR_S          0x53
-#define KEY_CHAR_T          0x54
-#define KEY_CHAR_U          0x55
-#define KEY_CHAR_V          0x56
-#define KEY_CHAR_W          0x57
-#define KEY_CHAR_X          0x58
-#define KEY_CHAR_Y          0x59
-#define KEY_CHAR_Z          0x5a
-
-
-// Control codes
-#define KEY_CTRL_FORMAT     30203
-#define KEY_CTRL_NOP        30202
-#define KEY_CTRL_EXE        30201
-#define KEY_CTRL_DEL        30025
-#define KEY_CTRL_AC         30070
-#define KEY_CTRL_FD         30046
-#define KEY_CTRL_UNDO	    30045	
-#define KEY_CTRL_XTT        30001
-#define KEY_CTRL_EXIT       5
-#define KEY_CTRL_OK       4
-#define KEY_CTRL_SHIFT      30006
-#define KEY_CTRL_ALPHA      30007
-#define KEY_CTRL_OPTN       30008
-#define KEY_CTRL_VARS       30030
-#define KEY_CTRL_UP         1
-#define KEY_CTRL_DOWN       2
-#define KEY_CTRL_LEFT       0
-#define KEY_CTRL_RIGHT      3
-#define KEY_CTRL_F1         30009
-#define KEY_CTRL_F2         30010
-#define KEY_CTRL_F3         30011
-#define KEY_CTRL_F4         30012
-#define KEY_CTRL_F5         30013
-#define KEY_CTRL_F6         30014
-#define KEY_CTRL_F7         30015
-#define KEY_CTRL_F8         30016
-#define KEY_CTRL_F9         30017
-#define KEY_CTRL_F10         30018
-#define KEY_CTRL_F11         30019
-#define KEY_CTRL_F12         30020
-#define KEY_CTRL_F13         30021
-#define KEY_CTRL_F14         30022
-#define KEY_CTRL_CATALOG    30100
-#define KEY_CTRL_CAPTURE    30055
-#define KEY_CTRL_CLIP       30050
-#define KEY_CTRL_CUT       30250
-#define KEY_CTRL_PASTE      30036
-#define KEY_CTRL_INS        30033
-#define KEY_CTRL_MIXEDFRAC  30054
-#define KEY_CTRL_FRACCNVRT  30026
-#define KEY_CTRL_QUIT       30029
-#define KEY_CTRL_PRGM       30028
-#define KEY_CTRL_SETUP      30037
-#define KEY_CTRL_PAGEUP     30052
-#define KEY_CTRL_PAGEDOWN   30053
-#define KEY_CTRL_MENU       30003
-#define KEY_SHIFT_OPTN	    30059
-#define KEY_CTRL_RESERVE1	30060
-#define KEY_CTRL_RESERVE2	30061
-#define KEY_SHIFT_LEFT		30062
-#define KEY_SHIFT_RIGHT		30063
-
-#define KEY_PRGM_ACON 10
-#define KEY_PRGM_DOWN 37
-#define KEY_PRGM_EXIT 47
-#define KEY_PRGM_F1 79
-#define KEY_PRGM_F2 69
-#define KEY_PRGM_F3 59
-#define KEY_PRGM_F4 49
-#define KEY_PRGM_F5 39
-#define KEY_PRGM_F6 29
-#define KEY_PRGM_LEFT 38
-#define KEY_PRGM_NONE 0
-#define KEY_PRGM_RETURN 31
-#define KEY_PRGM_RIGHT 27
-#define KEY_PRGM_UP 28
-#define KEY_PRGM_1 72
-#define KEY_PRGM_2 62
-#define KEY_PRGM_3 52
-#define KEY_PRGM_4 73
-#define KEY_PRGM_5 63
-#define KEY_PRGM_6 53
-#define KEY_PRGM_7 74
-#define KEY_PRGM_8 64
-#define KEY_PRGM_9 54
-#define KEY_PRGM_A 76
-#define KEY_PRGM_F 26
-#define KEY_PRGM_ALPHA 77 
-#define KEY_PRGM_SHIFT 78
-#define KEY_PRGM_MENU 48
 
 #endif // _KDISPLAY_H
 #endif
